@@ -1,5 +1,5 @@
 /*
- * $Id: subst.c 8593 2010-05-22 10:10:11Z dothebart $
+ * $Id$
  */
 #include "sysdep.h"
 
@@ -2208,9 +2208,9 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 	    (It->XPectContextType != TP->Filter.ContextType)) {
 		LogTemplateError(
 			Target, "Iterator", ERR_PARM1, TP,
-			"requires context of type %d, have %d", 
-			It->XPectContextType, 
-			TP->Filter.ContextType);
+			"requires context of type %s, have %s", 
+			ContextName(It->XPectContextType), 
+			ContextName(TP->Filter.ContextType));
 		return ;
 		
 	}
